@@ -65,7 +65,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <limits.h>
-#include "cdjpeg.h"
+#include "sane/sanei_jpeg.h"
 #include <sys/ioctl.h>
 
 #include "sane/sane.h"
@@ -358,10 +358,31 @@ init_dc210 (DC210 * camera)
 #ifdef HAVE_CFMAKERAW
   cfmakeraw (&tty_new);
 #else
-  tty_new.c_lflag &= ~(ICANON | ECHO | ISIG);
+  /* Modified to set the port REALLY as required. Code inspired by
+     the gPhoto2 serial port setup */
+
+  /* input control settings */
+  tty_new.c_iflag &= ~(IGNBRK | IGNCR | INLCR | ICRNL | IUCLC |
+                      IXANY | IXON | IXOFF | INPCK | ISTRIP);
+  tty_new.c_iflag |= (BRKINT | IGNPAR);
+  /* output control settings */
+  tty_new.c_oflag &= ~OPOST;
+  /* hardware control settings */
+  tty_new.c_cflag = (tty_new.c_cflag & ~CSIZE) | CS8;
+  tty_new.c_cflag &= ~(PARENB | PARODD | CSTOPB);
+# if defined(__sgi)
+  tty_new.c_cflag &= ~CNEW_RTSCTS;
+# else
+/* OS/2 doesn't have CRTSCTS - will this work for them? */
+#  ifdef CRTSCTS
+  tty_new.c_cflag &= ~CRTSCTS;
+#  endif
+# endif
+  tty_new.c_cflag |= CLOCAL | CREAD;
 #endif
-  tty_new.c_oflag &= ~CSTOPB;
-  tty_new.c_lflag = 0;
+  /* line discipline settings */
+  tty_new.c_lflag &= ~(ICANON | ISIG | ECHO | ECHONL | ECHOE |
+                       ECHOK | IEXTEN);
   tty_new.c_cc[VMIN] = 0;
   tty_new.c_cc[VTIME] = 5;
   cfsetospeed (&tty_new, B9600);
@@ -374,17 +395,30 @@ init_dc210 (DC210 * camera)
     }
 
   /* send a break to get it back to a known state */
+  /* Used to supply a non-zero argument to tcsendbreak(), TCSBRK, 
+   * and TCSBRKP, but that is system dependent.  e.g. on irix a non-zero
+   * value does a drain instead of a break.  A zero value is universally
+   * used to send a break.
+   */
+
 #ifdef HAVE_TCSENDBREAK
-  tcsendbreak (camera->fd, 4);
-#else
-# if defined(TCSBRKP)
-  ioctl (camera->fd, TCSBRKP, 4);
-# elif defined(TCSBRK)
-  ioctl (camera->fd, TCSBRK, 4);
+  tcsendbreak (camera->fd, 0);
+# if defined(__sgi)
+  tcdrain (camera->fd);
 # endif
+# elif defined(TCSBRKP)
+  ioctl (camera->fd, TCSBRKP, 0);
+# elif defined(TCSBRK)
+  ioctl (camera->fd, TCSBRK, 0);
 #endif
-  /* and wait for it to recover from the break */
+
+   /* and wait for it to recover from the break */
+
+#ifdef HAVE_USLEEP
   usleep (breakpause);
+#else
+  sleep (1);
+#endif
 
   if (send_pck (camera->fd, init_pck) == -1)
     {
@@ -875,7 +909,7 @@ sane_control_option (SANE_Handle handle, SANE_Int option,
        (action ==
 	SANE_ACTION_SET_VALUE ? "SET" : (action ==
 					 SANE_ACTION_GET_VALUE ? "GET" :
-					 "SETAUTO")), value, info);
+					 "SETAUTO")), value, (void *)info);
 
   if (handle != MAGIC || !is_open)
     return SANE_STATUS_INVAL;	/* Unknown handle ... */
