@@ -1,7 +1,7 @@
 /* sane - Scanner Access Now Easy.
 
    Copyright (C) 2002 Sergey Vlasov <vsu@altlinux.ru>
-   Copyright (C) 2002, 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2002 - 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
    
    This file is part of the SANE package.
    
@@ -156,8 +156,8 @@ gt68xx_device_new (GT68xx_Device ** dev_return)
 
   if (!dev)
     {
-      DBG (3, "gt68xx_device_new: couldn't malloc %d bytes for device\n",
-	   sizeof (GT68xx_Device));
+      DBG (3, "gt68xx_device_new: couldn't malloc %lu bytes for device\n",
+	   (u_long) sizeof (GT68xx_Device));
       *dev_return = 0;
       return SANE_STATUS_NO_MEM;
     }
@@ -174,6 +174,8 @@ gt68xx_device_new (GT68xx_Device ** dev_return)
   dev->read_buffer = NULL;
   dev->read_buffer_size = 32768;
 
+  dev->manual_selection = SANE_FALSE;
+
 #ifdef USE_FORK
   dev->shm_channel = NULL;
 #endif /* USE_FORK */
@@ -467,7 +469,8 @@ gt68xx_device_memory_write (GT68xx_Device * dev,
        (void *) dev, addr, size, data);
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_memory_write");
 
-  status = sanei_usb_control_msg (dev->fd, 0x40, 0x01,
+  status = sanei_usb_control_msg (dev->fd, 0x40,
+				  dev->model->command_set->request,
 				  dev->model->command_set->memory_write_value,
 				  addr, size, data);
 
@@ -476,7 +479,6 @@ gt68xx_device_memory_write (GT68xx_Device * dev,
       DBG (3, "gt68xx_device_memory_write: sanei_usb_control_msg failed: %s\n",
 	   sane_strstatus (status));
     }
-
   return status;
 }
 
@@ -490,7 +492,8 @@ gt68xx_device_memory_read (GT68xx_Device * dev,
        (void *) dev, addr, size, data);
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_memory_read");
 
-  status = sanei_usb_control_msg (dev->fd, 0xc0, 0x01,
+  status = sanei_usb_control_msg (dev->fd, 0xc0,
+				  dev->model->command_set->request,
 				  dev->model->command_set->memory_read_value,
 				  addr, size, data);
 
@@ -577,6 +580,9 @@ gt68xx_device_small_req (GT68xx_Device * dev, GT68xx_Packet cmd,
 				    fixed_cmd, res, 0x08);
 }
 
+
+#if 0
+/* currently not used */
 SANE_Status
 gt68xx_device_check_firmware (GT68xx_Device * dev, SANE_Bool * loaded)
 {
@@ -587,6 +593,7 @@ gt68xx_device_check_firmware (GT68xx_Device * dev, SANE_Bool * loaded)
   else
     return SANE_STATUS_UNSUPPORTED;
 }
+#endif
 
 SANE_Status
 gt68xx_device_download_firmware (GT68xx_Device * dev,
@@ -752,14 +759,13 @@ gt68xx_device_read_raw (GT68xx_Device * dev, SANE_Byte * buffer,
   DBG (7, "gt68xx_device_read_raw: enter: size=%lu\n", (unsigned long) *size);
 
   status = sanei_usb_read_bulk (dev->fd, buffer, size);
-
+  
   if (status != SANE_STATUS_GOOD)
     {
       DBG (3, "gt68xx_device_read_raw: bulk read failed: %s\n",
 	   sane_strstatus (status));
       return status;
     }
-
   DBG (7, "gt68xx_device_read_raw: leave: size=%lu\n", (unsigned long) *size);
 
   return SANE_STATUS_GOOD;
@@ -833,10 +839,10 @@ gt68xx_device_read_prepare (GT68xx_Device * dev, size_t expected_count,
 
 #ifdef USE_FORK
 
-static void
+static SANE_Status
 gt68xx_reader_process (GT68xx_Device * dev)
 {
-  SANE_Status status;
+  SANE_Status status = SANE_STATUS_GOOD;
   SANE_Int buffer_id;
   SANE_Byte *buffer_addr;
   size_t size;
@@ -873,8 +879,11 @@ gt68xx_reader_process (GT68xx_Device * dev)
       line++;
     }
   DBG (9, "gt68xx_reader_process: finished, now sleeping\n");
+  if (status != SANE_STATUS_GOOD)
+    return status;
   sleep (5 * 60);		/* wait until we are killed (or timeout) */
   shm_channel_writer_close (dev->shm_channel);
+  return status;
 }
 
 static SANE_Status
@@ -913,8 +922,8 @@ gt68xx_device_read_start_fork (GT68xx_Device * dev)
   if (pid == 0)
     {
       /* Child process */
-      gt68xx_reader_process (dev);
-      _exit (0);
+      status = gt68xx_reader_process (dev);
+      _exit (status);
     }
   else
     {
@@ -1034,6 +1043,8 @@ gt68xx_device_read (GT68xx_Device * dev, SANE_Byte * buffer, size_t * size)
 SANE_Status
 gt68xx_device_read_finish (GT68xx_Device * dev)
 {
+  SANE_Status status = SANE_STATUS_GOOD;
+
   CHECK_DEV_ACTIVE (dev, "gt68xx_device_read_finish");
 
   if (!dev->read_active)
@@ -1048,11 +1059,13 @@ gt68xx_device_read_finish (GT68xx_Device * dev)
 #ifdef USE_FORK
   if (dev->reader_pid != 0)
     {
-      int status;
+      int pid_status;
       /* usleep (100000);*/
       DBG (7, "gt68xx_device_read_finish: trying to kill reader process\n");
       kill (dev->reader_pid, SIGKILL);
-      waitpid (dev->reader_pid, &status, 0);
+      waitpid (dev->reader_pid, &pid_status, 0);
+      if (WIFEXITED(pid_status))
+	status = WEXITSTATUS(pid_status);
       DBG (7, "gt68xx_device_read_finish: reader process killed\n");
       dev->reader_pid = 0;
     }
@@ -1069,7 +1082,8 @@ gt68xx_device_read_finish (GT68xx_Device * dev)
 
   dev->read_active = SANE_FALSE;
 
-  return SANE_STATUS_GOOD;
+  DBG (7, "gt68xx_device_read_finish: exit (%s)\n", sane_strstatus (status));
+  return status;
 }
 
 static SANE_Status
