@@ -1,14 +1,9 @@
-/*.............................................................................
- * Project : SANE library for Plustek parallelport flatbed scanners.
- *.............................................................................
- */
-
 /* @file plustek-pp_misc.c
  * @brief here we have some helpful functions
 *
  * based on sources acquired from Plustek Inc.
  * Copyright (C) 1998 Plustek Inc.
- * Copyright (C) 2000-2003 Gerhard Jaeger <gerhard@gjaeger.de>
+ * Copyright (C) 2000-2004 Gerhard Jaeger <gerhard@gjaeger.de>
  * also based on the work done by Rick Bronson
  *
  * History:
@@ -34,6 +29,9 @@
  * - 0.41 - merged Kevins' patch to make EPP(ECP) work
  * - 0.42 - changed get_fast_time to _GET_TIME
  *        - changed include names
+ * - 0.43 - added LINUX_26 stuff
+ *        - minor fixes
+ *        - removed floating point stuff
  * .
  * <hr>
  * This file is part of the SANE package.
@@ -87,25 +85,22 @@
 # define PPA_PROBE_EPP19 0x0200
 #else
 
-/*
- * the parport driver in Kernel 2.4 has changed. It does report the
+/* the parport driver in Kernel 2.4 has changed. It does report the
  * possible modes in a different, more general way. As long, as
  * we do not use the parport-module change mode facility, I assume
  * the following correlations
  */
 #if defined LINUX_24 || defined LINUX_26
-# define PARPORT_MODE_PCPS2		PARPORT_MODE_TRISTATE
-# define PARPORT_MODE_PCEPP		PARPORT_MODE_EPP
-# define PARPORT_MODE_PCECPPS2	PARPORT_MODE_TRISTATE
+# define PARPORT_MODE_PCPS2     PARPORT_MODE_TRISTATE
+# define PARPORT_MODE_PCEPP     PARPORT_MODE_EPP
+# define PARPORT_MODE_PCECPPS2  PARPORT_MODE_TRISTATE
 # define PARPORT_MODE_PCECPEPP  PARPORT_MODE_EPP
-# define PARPORT_MODE_PCECR		PARPORT_MODE_ECP
+# define PARPORT_MODE_PCECR     PARPORT_MODE_ECP
 #endif
 #endif
 
-#define _A 16807         /* multiplier */
-#define _M 2147483647L   /* 2**31 - 1 */
-#define _Q 127773L       /* m div a */
-#define _R 2836          /* m mod a */
+#define _PP_A 16807         /**< multiplier */
+#define _PP_M 2147483647L   /**< 2**31 - 1  */
 
 /*************************** some local vars *********************************/
 
@@ -124,6 +119,36 @@ static int portIsClaimed[_MAX_PTDEVS] = { 0, 0, 0, 0 };
 /*************************** local functions *********************************/
 
 #ifdef __KERNEL__
+#ifdef LINUX_26
+
+static pScanData __ps = NULL;
+static int       __pa = -1;
+
+/** callback from parport driver
+ */
+static void misc_attach(struct parport *port)
+{
+	DBG( DBG_LOW, "misc_attach\n" );
+
+	__ps->pp = NULL;
+	if( port->base == (unsigned long)__pa ) {
+		DBG( DBG_LOW, "Requested port (0x%02x) found\n", __pa );
+		DBG( DBG_LOW, "Port mode reported: (0x%04x)\n",  port->modes );
+		__ps->pp = port;
+	}
+}
+
+static void misc_detach( struct parport *port )
+{
+	DBG( DBG_LOW, "misc_detach\n" );
+}
+
+static struct parport_driver pt_drv = {
+	.name   = "pt_drv",
+	.attach = misc_attach,
+	.detach = misc_detach,
+};
+#endif
 
 /** display the avaialable port-modes
  */
@@ -392,17 +417,17 @@ static Long miscNextLongRand( Long seed )
 {
 	ULong lo, hi;
 
-    lo = _A * (Long)(seed & 0xFFFF);
-    hi = _A * (Long)((ULong)seed >> 16);
+    lo = _PP_A * (Long)(seed & 0xFFFF);
+    hi = _PP_A * (Long)((ULong)seed >> 16);
     lo += (hi & 0x7FFF) << 16;
-    if (lo > _M) {
+    if (lo > _PP_M) {
 
-		lo &= _M;
+		lo &= _PP_M;
         ++lo;
 	}
     lo += hi >> 15;
-    if (lo > _M) {
-		lo &= _M;
+    if (lo > _PP_M) {
+		lo &= _PP_M;
         ++lo;
 	}
 
@@ -413,7 +438,7 @@ static Long miscNextLongRand( Long seed )
  */
 static void miscSeedLongRand( ULong seed )
 {
-	randomnum = seed ? (seed & _M) : 1;  /* nonzero seed */
+	randomnum = seed ? (seed & _PP_M) : 1;  /* nonzero seed */
 }
 
 /************************ exported functions *********************************/
@@ -442,12 +467,9 @@ _LOC int MiscReinitStruct( pScanData ps )
 		return _E_NULLPTR;
 
 	memset( ps, 0, sizeof(ScanData));
-		
- 	/*
-	 * first init all constant stuff in ScanData
-	 */
-	ps->sCaps.Version = ((_PTDRV_V1 << 8) | _PTDRV_V0);
 
+	/* first init all constant stuff in ScanData
+	 */
 	ps->bCurrentSpeed = 1;
 	ps->pbMapRed      =  ps->a_bMapTable;
 	ps->pbMapGreen    = &ps->a_bMapTable[256];
@@ -589,7 +611,7 @@ _LOC _INL void MiscStartTimer( pTimerDef timer , unsigned long us)
 	gettimeofday(&start_time, NULL);	
 #endif
 
-    *timer = start_time.tv_sec * 1e6 + start_time.tv_usec + us;
+    *timer = start_time.tv_sec * 1000000 + start_time.tv_usec + us;
 }
 
 /** Checks if a timer has been expired or not. In Kernel-mode, the scheduler
@@ -608,7 +630,7 @@ _LOC _INL int MiscCheckTimer( pTimerDef timer )
 	gettimeofday(&current_time, NULL);
 #endif
 
-    if (current_time.tv_sec * 1e6 + current_time.tv_usec > *timer) {
+    if (current_time.tv_sec * 1000000 + current_time.tv_usec > *timer) {
 		return _E_TIMEOUT;
     } else {
 #ifdef __KERNEL__       
@@ -655,11 +677,24 @@ _LOC int MiscRegisterPort( pScanData ps, int portAddr )
 	DBG( DBG_LOW, "Assigning port handle %i\n", portAddr );
     ps->pardev = portAddr;
 #else
-	struct parport *pp;
+
+#ifdef LINUX_26
+	__ps = ps;
+	__pa = portAddr;
 
 	DBG( DBG_LOW, "Requested port at 0x%02x\n", portAddr );
 
-	pp 		   = parport_enumerate();
+	if( parport_register_driver(&pt_drv)) {
+		/* Failed; nothing we can do. */
+		return _E_REGISTER;
+	}
+
+#else
+	struct parport *pp = NULL;
+
+	DBG( DBG_LOW, "Requested port at 0x%02x\n", portAddr );
+
+	pp         = parport_enumerate();
 	ps->pardev = NULL;
 
 	if( NULL == pp ) {
@@ -676,11 +711,12 @@ _LOC int MiscRegisterPort( pScanData ps, int portAddr )
 			ps->pp = pp;
 			break;
 		}
-
 		pp = pp->next;
 	}
+#endif
 
 	if( NULL == ps->pp ) {
+		printk("PORT not found!!!\n");
 		return _E_NO_PORT;
 	}
 
@@ -688,7 +724,7 @@ _LOC int MiscRegisterPort( pScanData ps, int portAddr )
 	 * register this device
 	 */
 	ps->pardev = parport_register_device( ps->pp, "Plustek Driver",
-						    miscPreemptionCallback, NULL, NULL, 0, (pVoid)ps );
+	                    miscPreemptionCallback, NULL, NULL, 0, (pVoid)ps );
 
 	if( NULL == ps->pardev ) {
 		return _E_REGISTER;
@@ -698,7 +734,6 @@ _LOC int MiscRegisterPort( pScanData ps, int portAddr )
 #endif
 
 	portIsClaimed[ps->devno] = 0;
-
 	return _OK;
 }
 
@@ -711,6 +746,9 @@ _LOC void MiscUnregisterPort( pScanData ps )
 		DBG( DBG_LOW, "Port unregistered\n" );
 		parport_unregister_device( ps->pardev );
 	}
+#ifdef LINUX_26
+	parport_unregister_driver( &pt_drv );
+#endif
 #else
 	sanei_pp_close( ps->pardev );
 #endif
