@@ -41,7 +41,7 @@
 */
 
 /* 
-	$Id: sceptre.h,v 1.5 2002/05/29 04:51:23 fzago Exp $
+	$Id: sceptre.h,v 1.6 2003/09/12 16:10:33 hmg-guest Exp $
 */
 
 /* Commands supported by the Sceptre S1200 scanner. */
@@ -268,14 +268,6 @@ enum Sceptre_Option
   OPT_NUM_OPTIONS
 };
 
-typedef union
-{
-  SANE_Word w;			/* word */
-  SANE_Word *wa;		/* word array */
-  SANE_String s;		/* string */
-}
-Option_Value;
-
 /*--------------------------------------------------------------------------*/
 
 /* 
