@@ -276,13 +276,14 @@ __SM3600EXPORT__ TState DoInit(TInstance *this);
 __SM3600EXPORT__ TState DoReset(TInstance *this);
 __SM3600EXPORT__ TState WaitWhileBusy(TInstance *this,int cSecs);
 __SM3600EXPORT__ TState WaitWhileScanning(TInstance *this,int cSecs);
-__SM3600EXPORT__ TState GetScannerModel(unsigned short idVendor, unsigned short idProduct);
+__SM3600EXPORT__ TModel GetScannerModel(unsigned short idVendor, unsigned short idProduct);
 
 #ifdef INSANE_VERSION
 __SM3600EXPORT__ TState DoLampSwitch(TInstance *this,int nPattern);
 #endif
 __SM3600EXPORT__ TState DoCalibration(TInstance *this);
 __SM3600EXPORT__ TState UploadGammaTable(TInstance *this, int iByteAddress, SANE_Int *pnGamma);
+__SM3600EXPORT__ TState UploadGainCorrection(TInstance *this, int iTableOffset);
 
 /* sm3600-scanusb.c */
 __SM3600EXPORT__ TState RegWrite(TInstance *this,int iRegister, int cb, unsigned long ulValue);
