@@ -47,7 +47,7 @@
    of the the AGFA SnapScan and Acer Vuego/Prisa flatbed scanners. */
 
 
-/* $Id: snapscan-scsi.c,v 1.26 2003/11/07 23:26:49 oliver-guest Exp $
+/* $Id: snapscan-scsi.c,v 1.29 2004/10/03 17:34:36 hmg-guest Exp $
    SnapScan backend scsi command functions */
 
 /* scanner scsi commands */
@@ -503,6 +503,7 @@ static SANE_Status inquiry (SnapScan_Scanner *pss)
     switch (pss->pdev->model)
     {
     case PERFECTION1670:
+    case PERFECTION2480:
         pss->bpp = 14;
         break;
     default:
@@ -759,7 +760,6 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
     u_short_to_u_charp (pss->res, pc + SET_WINDOW_P_YRES);
     DBG (DL_CALL_TRACE, "%s Resolution: %d\n", me, pss->res);
 
-    pos_factor = pss->actual_res;
     switch (pss->pdev->model)
     {
         case PRISA5000:
@@ -768,7 +768,11 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
         case PERFECTION1670:
             pos_factor = (pss->res > 800) ?  1600 : 800;
             break;
+        case PERFECTION2480:
+            pos_factor = (pss->res > 1200) ?  2400 : 1200;
+            break;
         default:
+            pos_factor = pss->actual_res;
             break;
     }
     /* it's an ugly sound if the scanner drives against the rear
@@ -861,7 +865,7 @@ static SANE_Status set_window (SnapScan_Scanner *pss)
        pss->pdev->model != PRISA610
     ) {
         pc[SET_WINDOW_P_DEBUG_MODE] = 2;        /* use full 128k buffer */
-        if ((pss->mode != MD_LINEART) && (pss->val[OPT_CUSTOM_GAMMA].b))
+        if (pss->mode != MD_LINEART)
         {
             pc[SET_WINDOW_P_GAMMA_NO] = 0x01;   /* downloaded gamma table */
         }
@@ -1059,7 +1063,7 @@ static SANE_Status read_calibration_data (SnapScan_Scanner *pss, void *buf, u_ch
     CHECK_STATUS (status, me, "snapscan_cmd");
 
     if(read_bytes != expected_read_bytes) {
-    DBG (DL_MAJOR_ERROR, "%s: read %d of %d calibration data\n", me, read_bytes, expected_read_bytes);
+    DBG (DL_MAJOR_ERROR, "%s: read %lu of %lu calibration data\n", me, (u_long) read_bytes, (u_long) expected_read_bytes);
     return SANE_STATUS_IO_ERROR;
     }
 
@@ -1169,6 +1173,7 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
                 fseek(fd, 0, SEEK_SET);
                 break;
             case PERFECTION1670:
+	    case PERFECTION2480:
                 /* Epson firmware files contain an info block which
                    specifies the length of the firmware data. The
                    length information is stored at offset 0x64 from
@@ -1198,7 +1203,7 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
                 break;
             }
 
-            DBG(DL_INFO, "Size of firmware: %d\n", bufLength);
+            DBG(DL_INFO, "Size of firmware: %lu\n", (u_long) bufLength);
             pCDB = (unsigned char *)malloc(bufLength + cdbLength);
             pFwBuf = pCDB + cdbLength;
             zero_buf (pCDB, cdbLength);
@@ -1222,6 +1227,15 @@ static SANE_Status download_firmware(SnapScan_Scanner * pss)
 
 /*
  * $Log: snapscan-scsi.c,v $
+ * Revision 1.29  2004/10/03 17:34:36  hmg-guest
+ * 64 bit platform fixes (bug #300799).
+ *
+ * Revision 1.28  2004/09/02 20:59:11  oliver-guest
+ * Added support for Epson 2480
+ *
+ * Revision 1.27  2004/04/02 20:19:24  oliver-guest
+ * Various bugfixes for gamma corretion (thanks to Robert Tsien)
+ *
  * Revision 1.26  2003/11/07 23:26:49  oliver-guest
  * Final bugfixes for bascic support of Epson 1670
  *
