@@ -1,4 +1,4 @@
-static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.4 2001/05/31 18:01:39 hmg Exp $";
+static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.c,v 1.10 2004/10/06 15:59:40 hmg-guest Exp $";
 /* sane - Scanner Access Now Easy.
 
    This file is part of the SANE package.
@@ -45,6 +45,25 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 
 /*
  * $Log: sp15c.c,v $
+ * Revision 1.10  2004/10/06 15:59:40  hmg-guest
+ * Don't eject medium twice after each page.
+ *
+ * Revision 1.9  2004/06/20 00:34:10  ellert-guest
+ * Missed one...
+ *
+ * Revision 1.8  2004/05/29 10:27:47  hmg-guest
+ * Fixed the fix of the sanei_thread fix (from Mattias Ellert).
+ *
+ * Revision 1.7  2004/05/28 18:52:53  hmg-guest
+ * Fixed sanei_thread fix (bug #300634, by Mattias Ellert).
+ *
+ * Revision 1.6  2004/05/23 17:28:56  hmg-guest
+ * Use sanei_thread instead of fork() in the unmaintained backends.
+ * Patches from Mattias Ellert (bugs: 300635, 300634, 300633, 300629).
+ *
+ * Revision 1.5  2003/12/27 17:48:38  hmg-guest
+ * Silenced some compilation warnings.
+ *
  * Revision 1.4  2001/05/31 18:01:39  hmg
  * Fixed config_line[len-1] bug which could generate an access
  * violation if len==0.
@@ -173,14 +192,13 @@ static const char RCSid[] = "$Header: /cvsroot/sane/sane-backends/backend/sp15c.
 #include <string.h>
 
 #include <sys/types.h>
-#include <sys/wait.h>
 #include <unistd.h>
 
 #include "sane/sanei_backend.h"
 #include "sane/sanei_scsi.h"
 #include "sane/saneopts.h"
 #include "sane/sanei_config.h"
-
+#include "sane/sanei_thread.h"
 
 #include "sp15c-scsi.h"
 #include "sp15c.h"
@@ -233,9 +251,13 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
   char dev_name[PATH_MAX];
   size_t len;
   FILE *fp;
+  authorize = authorize; /* silence compilation warnings */
+
   DBG_INIT ();
   DBG (10, "sane_init\n");
 
+  sanei_thread_init ();
+
   if (version_code)
     *version_code = SANE_VERSION_CODE (V_MAJOR, V_MINOR, 0);
   fp = sanei_config_open (SP15C_CONFIG_FILE);
@@ -266,6 +288,8 @@ sane_get_devices (const SANE_Device *** device_list, SANE_Bool local_only)
   static const SANE_Device **devlist = 0;
   struct sp15c *dev;
   int i;
+  
+  local_only = local_only; /* silence compilation warnings */
 
   DBG (10, "sane_get_devices\n");
 
@@ -290,6 +314,9 @@ sane_open (SANE_String_Const name, SANE_Handle * handle)
 {
   struct sp15c *dev = first_dev;
 
+  name = name; /* silence compilation warnings */
+  /* Strange, name is not used? */
+
   DBG (10, "sane_open\n");
 
   if (!dev)
@@ -340,6 +367,9 @@ sane_open (SANE_String_Const name, SANE_Handle * handle)
 SANE_Status
 sane_set_io_mode (SANE_Handle h, SANE_Bool non_blocking)
 {
+  h = h; 
+  non_blocking = non_blocking; /* silence compilation warnings */
+
   DBG (10, "sane_set_io_mode\n");
   return SANE_STATUS_UNSUPPORTED;
 }                               /* sane_set_io_mode */
@@ -348,6 +378,9 @@ sane_set_io_mode (SANE_Handle h, SANE_Bool non_blocking)
 SANE_Status
 sane_get_select_fd (SANE_Handle h, SANE_Int * fdp)
 {
+  h = h;
+  fdp = fdp; /* silence compilation warnings */
+
   DBG (10, "sane_get_select_fd\n");
   return SANE_STATUS_UNSUPPORTED;
 }                               /* sane_get_select_fd */
@@ -803,27 +836,12 @@ sane_start (SANE_Handle handle)
       return SANE_STATUS_IO_ERROR;
     }
 
-  scanner->reader_pid = fork ();
-  if (scanner->reader_pid == 0)
-    {
-      /* reader_pid = 0 ===> child process */
-      sigset_t ignore_set;
-      struct SIGACTION act;
-
-      close (fds[0]);
-
-      sigfillset (&ignore_set);
-      sigdelset (&ignore_set, SIGTERM);
-      sigprocmask (SIG_SETMASK, &ignore_set, 0);
-
-      memset (&act, 0, sizeof (act));
-      sigaction (SIGTERM, &act, 0);
-
-      /* don't use exit() since that would run the atexit() handlers... */
-      _exit (reader_process (scanner, fds[1]));
-    }
-  close (fds[1]);
   scanner->pipe = fds[0];
+  scanner->reader_pipe = fds[1];
+  scanner->reader_pid = sanei_thread_begin (reader_process, (void *) scanner);
+
+  if (sanei_thread_is_forked ())
+    close (scanner->reader_pipe);
 
   DBG (10, "sane_start: ok\n");
   return SANE_STATUS_GOOD;
@@ -1038,6 +1056,9 @@ attach_one (const char *name)
 static SANE_Status
 sense_handler (int scsi_fd, u_char * result, void *arg)
 {
+  scsi_fd = scsi_fd;
+  arg = arg; /* silence compilation warnings */
+
   return request_sense_parse (result);
 }                               /* sense_handler */
 
@@ -1273,6 +1294,7 @@ static SANE_Status
 sp15c_do_inquiry (struct sp15c *s)
 {
   static SANE_Status ret;
+  
   DBG (10, "do_inquiry\n");
 
   memset (s->buffer, '\0', 256);        /* clear buffer */
@@ -1283,7 +1305,7 @@ sp15c_do_inquiry (struct sp15c *s)
 }                               /* sp15c_do_inquiry */
 
 static SANE_Status
-do_scsi_cmd (int fd, char *cmd, int cmd_len, char *out, size_t out_len)
+do_scsi_cmd (int fd, unsigned char *cmd, int cmd_len, unsigned char *out, size_t out_len)
 {
   SANE_Status ret;
   size_t ol = out_len;
@@ -1548,10 +1570,14 @@ sp15c_free_scanner (struct sp15c *s)
 {
   int ret;
   DBG (10, "sp15c_free_scanner\n");
+#if 0
+  /* hmg: reports from several people show that this code ejects two pages
+     instead of one. So I've commented it out for now. */
   ret = sp15c_object_discharge (s);
   if (ret)
     return ret;
-
+#endif
+  
   wait_scanner (s);
 
   ret = do_scsi_cmd (s->sfd, release_unitB.cmd, release_unitB.size, NULL, 0);
@@ -1713,10 +1739,9 @@ do_cancel (struct sp15c *scanner)
       int exit_status;
       DBG (10, "do_cancel: kill reader_process\n");
       /* ensure child knows it's time to stop: */
-      kill (scanner->reader_pid, SIGTERM);
-      while (wait (&exit_status) != scanner->reader_pid)
-        DBG (50, "wait for scanner to stop\n");
-      ;
+      sanei_thread_kill (scanner->reader_pid);
+      DBG (50, "wait for scanner to stop\n");
+      sanei_thread_waitpid (scanner->reader_pid, &exit_status);
       scanner->reader_pid = 0;
     }
 
@@ -1733,7 +1758,10 @@ do_cancel (struct sp15c *scanner)
 
 static void
 swap_res (struct sp15c *s)
-{                               /* for the time being, do nothing */
+{
+  s = s; /* silence compilation warnings */
+  
+  /* for the time being, do nothing */
 }                               /* swap_res */
 
 static int
@@ -1763,6 +1791,8 @@ sp15c_set_window_param (struct sp15c *s, int prescan)
   int ret;
   int active_buffer_size;
 
+  prescan = prescan;   /* silence compilation warnings */
+  
   wait_scanner (s);
   DBG (10, "set_window_param\n");
   memset (buffer_r, '\0', WDB_size_max);        /* clear buffer */
@@ -1896,25 +1926,44 @@ sp15c_start_scan (struct sp15c *s)
 static void
 sigterm_handler (int signal)
 {
+  signal = signal; /* silence compilation warnings */
+
   sanei_scsi_req_flush_all ();  /* flush SCSI queue */
   _exit (SANE_STATUS_GOOD);
 }                               /* sigterm_handler */
 
 /* This function is executed as a child process. */
 static int
-reader_process (struct sp15c *scanner, int pipe_fd)
+reader_process (void *data)
 {
+  struct sp15c *scanner = (struct sp15c *) data;
+  int pipe_fd = scanner->reader_pipe;
+
   int status;
   unsigned int data_left;
   unsigned int data_to_read;
   FILE *fp;
+  sigset_t ignore_set;
   sigset_t sigterm_set;
   struct SIGACTION act;
-  int i;
+  unsigned int i;
   unsigned char *src, *dst;
 
   DBG (10, "reader_process started\n");
 
+  if (sanei_thread_is_forked ()) 
+    close (scanner->pipe);
+
+  sigfillset (&ignore_set);
+  sigdelset (&ignore_set, SIGTERM);
+#if defined (__APPLE__) && defined (__MACH__)
+  sigdelset (&ignore_set, SIGUSR2);
+#endif
+  sigprocmask (SIG_SETMASK, &ignore_set, 0);
+
+  memset (&act, 0, sizeof (act));
+  sigaction (SIGTERM, &act, 0);
+
   sigemptyset (&sigterm_set);
   sigaddset (&sigterm_set, SIGTERM);
 
@@ -2051,7 +2100,7 @@ bytes_per_line (struct sp15c *s)
 static void
 sp15c_trim_rowbufsize (struct sp15c *s)
 {
-  int row_len;
+  unsigned int row_len;
   row_len = bytes_per_line (s);
   if (s->row_bufsize >= row_len)
     {
