@@ -43,7 +43,7 @@
    This backend is for testing frontends.
 */
 
-#define BUILD 26
+#define BUILD 27
 
 #include "../include/sane/config.h"
 
@@ -453,7 +453,7 @@ init_options (Test_Device * test_device)
   od = &test_device->opt[opt_read_limit];
   od->name = "read-limit";
   od->title = SANE_I18N ("Read limit");
-  od->desc = SANE_I18N ("Limit the amount of data tranferred with each "
+  od->desc = SANE_I18N ("Limit the amount of data transferred with each "
 			"call to sane_read().");
   od->type = SANE_TYPE_BOOL;
   od->unit = SANE_UNIT_NONE;
@@ -685,7 +685,7 @@ init_options (Test_Device * test_device)
   od->type = SANE_TYPE_GROUP;
   od->unit = SANE_UNIT_NONE;
   od->size = 0;
-  od->cap = 0;
+  od->cap = SANE_CAP_ADVANCED;
   od->constraint_type = SANE_CONSTRAINT_NONE;
   od->constraint.range = 0;
   test_device->val[opt_bool_group].w = 0;
@@ -806,7 +806,7 @@ init_options (Test_Device * test_device)
   od->type = SANE_TYPE_GROUP;
   od->unit = SANE_UNIT_NONE;
   od->size = 0;
-  od->cap = 0;
+  od->cap = SANE_CAP_ADVANCED;
   od->constraint_type = SANE_CONSTRAINT_NONE;
   od->constraint.range = 0;
   test_device->val[opt_int_group].w = 0;
@@ -919,7 +919,7 @@ init_options (Test_Device * test_device)
   od->type = SANE_TYPE_GROUP;
   od->unit = SANE_UNIT_NONE;
   od->size = 0;
-  od->cap = 0;
+  od->cap = SANE_CAP_ADVANCED;
   od->constraint_type = SANE_CONSTRAINT_NONE;
   od->constraint.range = 0;
   test_device->val[opt_fixed_group].w = 0;
@@ -955,7 +955,7 @@ init_options (Test_Device * test_device)
     od->cap |= SANE_CAP_INACTIVE;
   od->constraint_type = SANE_CONSTRAINT_RANGE;
   od->constraint.range = &fixed_constraint_range;
-  test_device->val[opt_fixed_constraint_range].w = SANE_FIX (42.0);
+  test_device->val[opt_fixed_constraint_range].w = SANE_FIX (41.83);
 
   /* opt_fixed_constraint_word_list */
   od = &test_device->opt[opt_fixed_constraint_word_list];
@@ -1232,8 +1232,8 @@ reader_process (Test_Device * test_device, SANE_Int fd)
   if (status != SANE_STATUS_GOOD)
     return status;
 
-  DBG (2, "(child) reader_process: buffer=%p, buffersize=%d\n",
-       buffer, buffer_size);
+  DBG (2, "(child) reader_process: buffer=%p, buffersize=%lu\n",
+       buffer, (u_long) buffer_size);
 
   while (byte_count < bytes_total)
     {
@@ -1254,18 +1254,27 @@ reader_process (Test_Device * test_device, SANE_Int fd)
 	  return SANE_STATUS_IO_ERROR;
 	}
       byte_count += bytes_written;
-      DBG (4, "(child) reader_process: wrote %d bytes of %d (%d total)\n",
-	   bytes_written, write_count, byte_count);
+      DBG (4, "(child) reader_process: wrote %ld bytes of %lu (%d total)\n",
+	   (long) bytes_written, (u_long) write_count, byte_count);
       write_count -= bytes_written;
     }
 
   free (buffer);
-  DBG (4, "(child) reader_process: finished,  wrote %d bytes, expected %d "
+
+  if (sanei_thread_is_forked ())
+    {
+	  DBG (4, "(child) reader_process: finished,  wrote %d bytes, expected %d "
        "bytes, now waiting\n", byte_count, bytes_total);
-  while (SANE_TRUE)
-    sleep (10);
-  DBG (4, "(child) reader_process: this should have never happened...");
-  close (fd);
+	  while (SANE_TRUE)
+	    sleep (10);
+	  DBG (4, "(child) reader_process: this should have never happened...");
+	  close (fd);
+    }
+  else
+    {
+	  DBG (4, "(child) reader_process: finished,  wrote %d bytes, expected %d "
+       "bytes\n", byte_count, bytes_total);
+    }
   return SANE_STATUS_GOOD;
 }
 
@@ -1308,12 +1317,12 @@ finish_pass (Test_Device * test_device)
 
   DBG (2, "finish_pass: test_device=%p\n", (void *) test_device);
   test_device->scanning = SANE_FALSE;
-  if (test_device->pipe > 0)
+  if (test_device->pipe >= 0)
     {
       DBG (2, "finish_pass: closing pipe\n");
       close (test_device->pipe);
       DBG (2, "finish_pass: pipe closed\n");
-      test_device->pipe = 0;
+      test_device->pipe = -1;
     }
   if (test_device->reader_pid > 0)
     {
@@ -1337,6 +1346,14 @@ finish_pass (Test_Device * test_device)
 	}
       test_device->reader_pid = 0;
     }
+  /* this happens when running in thread context... */
+  if (test_device->reader_fds >= 0)
+    {
+      DBG (2, "finish_pass: closing reader pipe\n");
+      close (test_device->reader_fds);
+      DBG (2, "finish_pass: reader pipe closed\n");
+      test_device->reader_fds = -1;
+    }
   return return_status;
 }
 
@@ -1591,8 +1608,8 @@ sane_init (SANE_Int * version_code, SANE_Auth_Callback authorize)
       test_device->eof = SANE_FALSE;
       test_device->scanning = SANE_FALSE;
       test_device->cancelled = SANE_FALSE;
-      test_device->reader_pid = 0;
-      test_device->pipe = 0;
+      test_device->reader_pid = -1;
+      test_device->pipe = -1;
       DBG (4, "sane_init: new device: `%s' is a %s %s %s\n",
 	   test_device->sane.name, test_device->sane.vendor,
 	   test_device->sane.model, test_device->sane.type);
@@ -2636,7 +2653,7 @@ sane_read (SANE_Handle handle, SANE_Byte * data,
   *length = bytes_read;
   test_device->bytes_total += bytes_read;
 
-  DBG (2, "sane_read: read %d bytes of %d, total %d\n", bytes_read,
+  DBG (2, "sane_read: read %ld bytes of %d, total %d\n", (long) bytes_read,
        max_scan_length, test_device->bytes_total);
   return SANE_STATUS_GOOD;
 }
