@@ -262,15 +262,15 @@ reset_watchdog (void)
 
 static void
 auth_callback (SANE_String_Const res,
-	       SANE_Char username[SANE_MAX_USERNAME_LEN],
-	       SANE_Char password[SANE_MAX_PASSWORD_LEN])
+	       SANE_Char *username,
+	       SANE_Char *password)
 {
   SANE_Net_Procedure_Number procnum;
   SANE_Authorization_Req req;
   SANE_Word word, ack = 0;
 
-  memset (username, 0, sizeof (username));
-  memset (password, 0, sizeof (password));
+  memset (username, 0, SANE_MAX_USERNAME_LEN);
+  memset (password, 0, SANE_MAX_PASSWORD_LEN);
 
   if (!can_authorize)
     {
@@ -1715,6 +1715,14 @@ process_request (Wire * w)
 	    return;
 	  }
 
+	if (!name)
+	  {
+	    DBG (DBG_ERR, "process_request: (open) device_name == NULL\n");
+	    reply.status = SANE_STATUS_INVAL;
+	    sanei_w_reply (w, (WireCodecFunc) sanei_w_open_reply, &reply);
+	    return;
+	  }
+
 	can_authorize = 1;
 
 	resource = strdup (name);
@@ -2074,7 +2082,7 @@ main (int argc, char *argv[])
       DBG (DBG_WARN, "main: starting debug mode (level %d)\n", debug);
 
       DBG (DBG_DBG, 
-	   "main: trying to get port for service `sane' (getaddrinfo)\n");
+	   "main: trying to get port for service `sane-port' (getaddrinfo)\n");
 
       memset (&hints, 0, sizeof (struct addrinfo));
 
@@ -2082,11 +2090,11 @@ main (int argc, char *argv[])
       hints.ai_flags = AI_PASSIVE;
       hints.ai_socktype = SOCK_STREAM;
 
-      err = getaddrinfo (NULL, "sane", &hints, &res);
+      err = getaddrinfo (NULL, "sane-port", &hints, &res);
       if (err)
 	{
-	  DBG (DBG_WARN, "main: \"sane\" service unknown on your host; you should add\n");
-	  DBG (DBG_WARN, "main:      sane 6566/tcp saned # SANE network scanner daemon\n");
+	  DBG (DBG_WARN, "main: \"sane-port\" service unknown on your host; you should add\n");
+	  DBG (DBG_WARN, "main:      sane-port 6566/tcp saned # SANE network scanner daemon\n");
 	  DBG (DBG_WARN, "main: to your /etc/services file (or equivalent). Proceeding anyway.\n");
 	  err = getaddrinfo (NULL, "6566", &hints, &res);
 	  if (err)
@@ -2326,7 +2334,7 @@ main (int argc, char *argv[])
 
       DBG (DBG_DBG, 
 	   "main: trying to get port for service `sane' (getservbyname)\n");
-      serv = getservbyname ("sane", "tcp");
+      serv = getservbyname ("sane-port", "tcp");
       if (serv)
 	{
 	  port = serv->s_port;
