@@ -1,6 +1,6 @@
 /* sane - Scanner Access Now Easy.
    Copyright (C) 1998-2001 Yuri Dario
-   Copyright (C) 2003 Gerhard Jaeger (pthread/process support)
+   Copyright (C) 2003-2004 Gerhard Jaeger (pthread/process support)
    This file is part of the SANE package.
 
    This program is free software; you can redistribute it and/or
@@ -117,7 +117,11 @@ sanei_thread_kill( int pid )
 {
 	DBG(2, "sanei_thread_kill() will kill %d\n", (int)pid);
 #ifdef USE_PTHREAD
+#if defined (__APPLE__) && defined (__MACH__)
+	return pthread_kill((pthread_t)pid, SIGUSR2);
+#else
 	return pthread_cancel((pthread_t)pid);
+#endif
 #elif defined HAVE_OS2_H
 	return DosKillThread(pid);
 #else
@@ -189,19 +193,43 @@ sanei_thread_sendsig( int pid, int sig )
 
 /**
  */
+#if defined (__APPLE__) && defined (__MACH__)
+static void
+thread_exit_handler( int signo )
+{
+	DBG( 2, "signal(%i) caught, calling pthread_exit now...\n", signo );
+	pthread_exit( PTHREAD_CANCELED );
+}
+#endif
+
+
 static void*
 local_thread( void *arg )
 {
 	static int     status;
-	int            old;
 	pThreadDataDef ltd = (pThreadDataDef)arg;
 
-	DBG( 2, "thread started, calling func() now...\n" );
+#if defined (__APPLE__) && defined (__MACH__)
+	struct sigaction act;
+
+	sigemptyset(&(act.sa_mask));
+	act.sa_flags   = 0;
+	act.sa_handler = thread_exit_handler;
+	sigaction( SIGUSR2, &act, 0 );
+#else
+	int old;
+
 	pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, &old );
-	pthread_setcanceltype( PTHREAD_CANCEL_ASYNCHRONOUS, &old );
-	
+	pthread_setcanceltype ( PTHREAD_CANCEL_ASYNCHRONOUS, &old );
+#endif
+
+	DBG( 2, "thread started, calling func() now...\n" );
+
 	status = ltd->func( ltd->func_data );
 
+	/* so sanei_thread_get_status() will work correctly... */
+	ltd->status = status;
+
 	DBG( 2, "func() done - status = %d\n", status );
 
 	/* return the status, so pthread_join is able to get it*/
@@ -275,10 +303,11 @@ sanei_thread_begin( int (func)(void *args), void* args )
 		}
 	}
 
-    td.func      = func;
-    td.func_data = args;
+	td.func      = func;
+	td.func_data = args;
 
 	pid = pthread_create( &thread, NULL, local_thread, &td );
+	usleep( 1 );
 
 	if ( pid != 0 ) {
 		DBG( 1, "pthread_create() failed with %d\n", pid );
@@ -332,7 +361,7 @@ sanei_thread_waitpid( int pid, int *status )
 
 	stat = 0;
 
-	DBG(2, "sanei_thread_waitpid() - %d\n", (int)pid);
+	DBG(2, "sanei_thread_waitpid() - %d\n", pid);
 #ifdef USE_PTHREAD
 	result = pthread_join((pthread_t)pid, (void*)&ls );
 
@@ -344,14 +373,17 @@ sanei_thread_waitpid( int pid, int *status )
 			stat = *ls;
 		}
 		DBG(2, "* result = %d (%p)\n", stat, (void*)status );
+		result = pid;
 	}
+	/* call detach in any case to make sure that the thread resources 
+	 * will be freed, when the thread has terminated
+	 */
+	DBG(2, "* detaching thread(%d)\n", pid );
+	pthread_detach((pthread_t)pid);
 	if (status)
 		*status = stat;
 
 	restore_sigpipe();
-		
-	/* should return pid */
-	return pid;
 #else
 	result = waitpid( pid, &ls, 0 );
 	if((result < 0) && (errno == ECHILD)) {
@@ -363,9 +395,8 @@ sanei_thread_waitpid( int pid, int *status )
 	}
 	if( status )
 		*status = stat;
-
-	return result;
 #endif
+	return result;
 }
 
 #endif /* HAVE_OS2_H */
