@@ -74,8 +74,8 @@ sanei_w_space (Wire * w, size_t howmuch)
 
   if (w->buffer.curr + howmuch > w->buffer.end)
     {
-      DBG (4, "sanei_w_space: free buffer size is %d\n",
-	   w->buffer.end - w->buffer.curr);
+      DBG (4, "sanei_w_space: free buffer size is %lu\n",
+	   (u_long) (w->buffer.end - w->buffer.curr));
       switch (w->direction)
 	{
 	case WIRE_ENCODE:
@@ -97,8 +97,8 @@ sanei_w_space (Wire * w, size_t howmuch)
 	    }
 	  w->buffer.curr = w->buffer.start;
 	  w->buffer.end = w->buffer.start + w->buffer.size;
-	  DBG (4, "sanei_w_space: ENCODE: free buffer is now %d\n",
-	       w->buffer.size);
+	  DBG (4, "sanei_w_space: ENCODE: free buffer is now %lu\n",
+	       (u_long) w->buffer.size);
 	  break;
 
 	case WIRE_DECODE:
@@ -137,8 +137,8 @@ sanei_w_space (Wire * w, size_t howmuch)
 	      w->buffer.end += nread;
 	    }
 	  while (left_over < howmuch);
-	  DBG (4, "sanei_w_space: DECODE: %d bytes read\n",
-	       w->buffer.end - w->buffer.start);
+	  DBG (4, "sanei_w_space: DECODE: %lu bytes read\n",
+	       (u_long) (w->buffer.end - w->buffer.start));
 	  break;
 
 	case WIRE_FREE:
@@ -179,6 +179,7 @@ sanei_w_array (Wire * w, SANE_Word * len_ptr, void **v,
 	      val += element_size;
 	    }
 	  free (*v);
+	  w->allocated_memory -= (*len_ptr * element_size);
 	}
       else
 	DBG (1, "sanei_w_array: FREE: tried to free array but *len_ptr or *v "
@@ -192,13 +193,29 @@ sanei_w_array (Wire * w, SANE_Word * len_ptr, void **v,
     len = *len_ptr;
   DBG (4, "sanei_w_array: send/receive array length\n");
   sanei_w_word (w, &len);
-  DBG (4, "sanei_w_array: array has %d elements\n", len);
 
+  if (w->status)
+    {
+      DBG (1, "sanei_w_array: bad status: %d\n", w->status);
+      return;
+    }
+  DBG (4, "sanei_w_array: array has %d elements\n", len);
+      
   if (w->direction == WIRE_DECODE)
     {
       *len_ptr = len;
       if (len)
 	{
+	  if (((unsigned int) len) > MAX_MEM 
+	      || ((unsigned int) len * element_size) > MAX_MEM 
+	      || (w->allocated_memory + len * element_size) > MAX_MEM)
+	    {
+	      DBG (0, "sanei_w_array: DECODE: maximum amount of allocated memory "
+		   "exceeded (limit: %u, new allocation: %u, total: %u bytes)\n",
+		   MAX_MEM, len * element_size, MAX_MEM + len * element_size);
+	      w->status = ENOMEM;
+	      return;
+	    }
 	  *v = malloc (len * element_size);
 	  if (*v == 0)
 	    {
@@ -208,6 +225,7 @@ sanei_w_array (Wire * w, SANE_Word * len_ptr, void **v,
 	      return;
 	    }
 	  memset (*v, 0, len * element_size);
+	  w->allocated_memory += (len * element_size);
 	}
       else
 	*v = 0;
@@ -219,6 +237,11 @@ sanei_w_array (Wire * w, SANE_Word * len_ptr, void **v,
     {
       (*w_element) (w, val);
       val += element_size;
+      if (w->status)
+	{
+	  DBG (1, "sanei_w_array: bad status: %d\n", w->status);
+	  return;
+	}
     }
   DBG (4, "sanei_w_array: done\n");
 }
@@ -238,6 +261,7 @@ sanei_w_ptr (Wire * w, void **v, WireCodecFunc w_value, size_t value_size)
 	  DBG (4, "sanei_w_ptr: FREE: freeing value\n");
 	  (*w_value) (w, *v);
 	  free (*v);
+	  w->allocated_memory -= value_size;
 	}
       else
 	DBG (1, "sanei_w_ptr: FREE: tried to free value but *v or value_size "
@@ -251,12 +275,26 @@ sanei_w_ptr (Wire * w, void **v, WireCodecFunc w_value, size_t value_size)
 
   DBG (4, "sanei_w_ptr: send/receive is_null\n");
   sanei_w_word (w, &is_null);
+  if (w->status)
+    {
+      DBG (1, "sanei_w_ptr: bad status: %d\n", w->status);
+      return;
+    }
 
   if (!is_null)
     {
       if (w->direction == WIRE_DECODE)
 	{
 	  DBG (4, "sanei_w_ptr: DECODE: receive data pointed at\n");
+	  if (value_size > MAX_MEM)
+	    {
+	      DBG (0, "sanei_w_ptr: DECODE: maximum amount of allocated memory "
+		   "exceeded (limit: %u, new allocation: %u, total: %u bytes)\n",
+		   MAX_MEM, value_size, (w->allocated_memory + value_size));
+	      w->status = ENOMEM;
+	      return;
+	    }
+
 	  *v = malloc (value_size);
 	  if (*v == 0)
 	    {
@@ -265,6 +303,7 @@ sanei_w_ptr (Wire * w, void **v, WireCodecFunc w_value, size_t value_size)
 	      w->status = ENOMEM;
 	      return;
 	    }
+	  w->allocated_memory += value_size;
 	  memset (*v, 0, value_size);
 	}
       (*w_value) (w, *v);
@@ -307,7 +346,7 @@ sanei_w_string (Wire * w, SANE_String * v)
 {
   DBG (3, "sanei_w_string: wire %d\n", w->io.fd);
   (*w->codec.w_string) (w, v);
-  if (w->direction != WIRE_FREE)
+  if (w->direction != WIRE_FREE && w->status == 0)
     DBG (4, "sanei_w_string: value = %s\n", *v);
 }
 
@@ -544,8 +583,8 @@ sanei_w_set_dir (Wire * w, WireDirection dir)
        w->direction == WIRE_ENCODE ? "ENCODE" : 
        (w->direction == WIRE_DECODE ? "DECODE" : "FREE"));
   if (w->direction == WIRE_DECODE && w->buffer.curr != w->buffer.end)
-    DBG (1, "sanei_w_set_dir: WARNING: will delete %d bytes from buffer\n",
-	 w->buffer.end - w->buffer.curr);
+    DBG (1, "sanei_w_set_dir: WARNING: will delete %lu bytes from buffer\n",
+	 (u_long) (w->buffer.end - w->buffer.curr));
   flush (w);
   w->direction = dir;
   DBG (4, "sanei_w_set_dir: direction changed\n");
@@ -621,7 +660,7 @@ sanei_w_init (Wire * w, void (*codec_init_func) (Wire *))
   w->direction = WIRE_ENCODE;
   w->buffer.size = 8192;
   w->buffer.start = malloc (w->buffer.size);
-
+  
   if (w->buffer.start == 0)
     {
       /* Malloc failed, so return an error. */
@@ -636,6 +675,7 @@ sanei_w_init (Wire * w, void (*codec_init_func) (Wire *))
       DBG (4, "sanei_w_init: initializing codec\n");
       (*codec_init_func) (w);
     }
+  w->allocated_memory = 0;
   DBG (4, "sanei_w_init: done\n");
 }
 
