@@ -1,9 +1,11 @@
 /*
    check-usb-chip.c -- Find out what USB scanner chipset is used
    
-   Copyright (C) 2003 Henning Meier-Geinitz <henning@meier-geinitz.de>
+   Copyright (C) 2003, 2004 Henning Meier-Geinitz <henning@meier-geinitz.de>
    Copyright (C) 2003 Gerhard Jger <gerhard@gjaeger.de>
                       for LM983x tests
+   Copyright (C) 2003 Gerard Klaver <gerard at gkall dot hobby dot nl>
+                      for ICM532B tests
 
    This file is part of the SANE package.
 
@@ -25,19 +27,21 @@
 
 
 #include "../include/sane/config.h"
+#include "../include/sane/sane.h"
 
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 
 static int verbose = 0;
-
+static SANE_Bool no_chipset_access;
 #define TIMEOUT 1000
 
 #ifdef HAVE_LIBUSB
 #include <usb.h>
 
-extern char *check_usb_chip (struct usb_device *dev, int verbosity);
+extern char *check_usb_chip (struct usb_device *dev, int verbosity,
+			     SANE_Bool from_file);
 
 
 static int
@@ -45,6 +49,9 @@ prepare_interface (struct usb_device *dev, usb_dev_handle ** handle)
 {
   int result;
 
+  if (no_chipset_access)
+    return 0;
+
   *handle = usb_open (dev);
   if (*handle == 0)
     {
@@ -239,6 +246,14 @@ check_gt6816 (struct usb_device *dev)
       return 0;
     }
 
+  if (dev->config[0].bNumInterfaces != 0x01)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GT-6816 (bNumInterfaces = 0x%x)\n",
+		dev->config[0].bNumInterfaces);
+      return 0;
+    }
+
   /* Check endpoints */
   if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 2)
     {
@@ -317,11 +332,19 @@ check_gt6816 (struct usb_device *dev)
     usb_control_msg (handle, 0xc0, 0x01, 0x2013, 0x3f00, req, 64, TIMEOUT);
   if (result <= 0)
     {
-      if (verbose > 2)
-	printf ("    Couldn't send read control message (%s)\n",
-		strerror (errno));
-      finish_interface (handle);
-      return 0;
+      /* Before firmware upload, 64 bytes are returned. Some libusb
+	 implementations/operating systems can't seem to cope with short
+	 packets. */
+      result =
+	usb_control_msg (handle, 0xc0, 0x01, 0x2013, 0x3f00, req, 8, TIMEOUT);
+      if (result <= 0)
+	{
+	  if (verbose > 2)
+	    printf ("    Couldn't send read control message (%s)\n",
+		    strerror (errno));
+	  finish_interface (handle);
+	  return 0;
+	}
     }
 
   if (req[0] != 0)
@@ -336,6 +359,206 @@ check_gt6816 (struct usb_device *dev)
   return "GT-6816";
 }
 
+/* Check for Grandtech GT-8911 */
+static char *
+check_gt8911 (struct usb_device *dev)
+{
+  char req[64];
+  usb_dev_handle *handle;
+  int result;
+
+  if (verbose > 2)
+    printf ("    checking for GT-8911 ...\n");
+
+  /* Check device descriptor */
+  if ((dev->descriptor.bDeviceClass != USB_CLASS_PER_INTERFACE)
+      || (dev->config[0].interface[0].altsetting[0].bInterfaceClass !=
+	  USB_CLASS_VENDOR_SPEC))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 1, bDeviceClass = %d, bInterfaceClass = %d)\n",
+	   dev->descriptor.bDeviceClass,
+	   dev->config[0].interface[0].altsetting[0].bInterfaceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x110)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GT-8911 (check 2, bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0x00)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 3, bDeviceSubClass = 0x%x)\n",
+	   dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0x00)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 4, bDeviceProtocol = 0x%x)\n",
+	   dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 2)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GT-8911 (check 5, bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+  /* Check first endpoint descriptor block */
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x81)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 6, bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+       bEndpointAddress != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 7, bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+      return 0;
+
+    }
+  if (dev->config[0].bNumInterfaces < 2)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GT-8911 (check 8, bNumInterfaces = %d)\n",
+		dev->config[0].bNumInterfaces);
+      return 0;
+    }
+  if (dev->config[0].interface[1].num_altsetting < 3)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GT-8911 (check 9, num_altsetting = %d)\n",
+		dev->config[0].interface[1].num_altsetting);
+      return 0;
+    }
+
+  /* Check fourth endpoint descriptor block */
+  if ((dev->config[0].interface[1].altsetting[2].endpoint[0].
+       bEndpointAddress != 0x83)
+      || (dev->config[0].interface[1].altsetting[2].endpoint[0].
+	  bmAttributes != 0x01)
+      || (dev->config[0].interface[1].altsetting[2].endpoint[0].
+	  wMaxPacketSize != 0x01d0)
+      || (dev->config[0].interface[1].altsetting[2].endpoint[0].bInterval !=
+	  0x01))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 10, bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[1].altsetting[2].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[1].altsetting[2].endpoint[0].bmAttributes,
+	   dev->config[0].interface[1].altsetting[2].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[1].altsetting[2].endpoint[0].bInterval);
+      return 0;
+    }
+  if ((dev->config[0].interface[1].altsetting[2].endpoint[1].
+       bEndpointAddress != 0x04)
+      || (dev->config[0].interface[1].altsetting[2].endpoint[1].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[1].altsetting[2].endpoint[1].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[1].altsetting[2].endpoint[1].bInterval !=
+	  0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 11, bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[1].altsetting[2].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[1].altsetting[2].endpoint[1].bmAttributes,
+	   dev->config[0].interface[1].altsetting[2].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[1].altsetting[2].endpoint[1].bInterval);
+      return 0;
+
+    }
+
+  /* Now we send a control message */
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "GT-8911?";
+
+  memset (req, 0, 8);
+  req[0] = 0x55;
+  req[1] = 0x66;
+
+  result =
+    usb_control_msg (handle, 0xc0, 0x10, 0x41, 0x0000, req, 64, TIMEOUT);
+  if (result <= 0)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 12, couldn't send read control message (%s))\n",
+	   strerror (errno));
+      finish_interface (handle);
+      return 0;
+    }
+  result =
+    usb_control_msg (handle, 0xc0, 0x10, 0x05, 0x0000, req, 64, TIMEOUT);
+  if (result <= 0)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GT-8911 (check 13, couldn't send read control message (%s)\n",
+	   strerror (errno));
+      finish_interface (handle);
+      return 0;
+    }
+  /* tested on model hardware version 0xffffffc0, firmware version 0x10)) */
+  if (verbose > 2)
+    printf
+      ("    Check 14, control message (hardware version %0x / firmware version %0x)\n",
+       req[0], req[1]);
+
+  finish_interface (handle);
+  return "GT-8911";
+}
+
 /* Check for Mustek MA-1017 */
 static char *
 check_ma1017 (struct usb_device *dev)
@@ -477,6 +700,10 @@ check_ma1017 (struct usb_device *dev)
       finish_interface (handle);
       return 0;
     }
+  /* Read one byte again to work around a bug in the MA-1017 chipset that 
+     appears when an odd number of bytes is read or written. */
+  result = usb_bulk_write (handle, 0x01, req, 2, 1000);
+  result = usb_bulk_read (handle, 0x82, &res, 1, 1000);
   finish_interface (handle);
   return "MA-1017";
 }
@@ -664,20 +891,17 @@ check_ma1509 (struct usb_device *dev)
       || (dev->config[0].interface[0].altsetting[0].endpoint[0].
 	  bmAttributes != 0x02)
       || (dev->config[0].interface[0].altsetting[0].endpoint[0].
-	  wMaxPacketSize != 0x40)
-      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
-	  0xff))
+	  wMaxPacketSize != 0x40))
     {
       if (verbose > 2)
 	printf
 	  ("    this is not a MA-1509 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
-	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   "wMaxPacketSize = 0x%x)\n",
 	   dev->config[0].interface[0].altsetting[0].endpoint[0].
 	   bEndpointAddress,
 	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
 	   dev->config[0].interface[0].altsetting[0].endpoint[0].
-	   wMaxPacketSize,
-	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+	   wMaxPacketSize);
       return 0;
     }
   if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
@@ -685,20 +909,17 @@ check_ma1509 (struct usb_device *dev)
       || (dev->config[0].interface[0].altsetting[0].endpoint[1].
 	  bmAttributes != 0x02)
       || (dev->config[0].interface[0].altsetting[0].endpoint[1].
-	  wMaxPacketSize != 0x08)
-      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
-	  0x00))
+	  wMaxPacketSize != 0x08))
     {
       if (verbose > 2)
 	printf
 	  ("    this is not a MA-1509 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
-	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   "wMaxPacketSize = 0x%x)\n",
 	   dev->config[0].interface[0].altsetting[0].endpoint[1].
 	   bEndpointAddress,
 	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
 	   dev->config[0].interface[0].altsetting[0].endpoint[1].
-	   wMaxPacketSize,
-	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+	   wMaxPacketSize);
       return 0;
     }
   if ((dev->config[0].interface[0].altsetting[0].endpoint[2].
@@ -1181,6 +1402,160 @@ check_gl646 (struct usb_device *dev)
   return "GL646";
 }
 
+/* Same as check_gl646, except that sanity check are different. */
+static char *
+check_gl646_hp (struct usb_device *dev)
+{
+  unsigned char val;
+  int result;
+  usb_dev_handle *handle;
+
+  if (verbose > 2)
+    printf ("    checking for GL646_HP ...\n");
+
+  /* Check device descriptor */
+  if ((dev->descriptor.bDeviceClass != 0xff)
+      || (dev->config[0].interface[0].altsetting[0].bInterfaceClass != 0xff))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL646_HP (bDeviceClass = %d, bInterfaceClass = %d)\n",
+	   dev->descriptor.bDeviceClass,
+	   dev->config[0].interface[0].altsetting[0].bInterfaceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x110)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL646_HP (bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0xff)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL646_HP (bDeviceSubClass = 0x%x)\n",
+		dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0xff)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL646_HP (bDeviceProtocol = 0x%x)\n",
+		dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 3)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL646_HP (bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x81)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x0))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL646_HP (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+       bEndpointAddress != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  bmAttributes != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  wMaxPacketSize != 0x40)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
+	  0))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL646_HP (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[2].
+       bEndpointAddress != 0x83)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  bmAttributes != 0x03)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  wMaxPacketSize != 0x1)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval !=
+	  8))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL646_HP (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval);
+      return 0;
+    }
+
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "GL646_HP?";
+
+  result = gl646_write_reg (handle, 0x38, 0x15);
+  if (!result)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL646_HP (writing register failed)\n");
+      finish_interface (handle);
+      return 0;
+    }
+
+  result = gl646_read_reg (handle, 0x4e, &val);
+  if (!result)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL646_HP (reading register failed)\n");
+      finish_interface (handle);
+      return 0;
+    }
+
+  if (val != 0x15)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL646_HP (reg 0x4e != reg 0x38)\n");
+      finish_interface (handle);
+      return 0;
+    }
+
+  finish_interface (handle);
+
+  return "GL646_HP";
+}
+
 /* check for the combination of gl660 and gl646 */
 static char *
 check_gl660_gl646 (struct usb_device *dev)
@@ -1194,7 +1569,8 @@ check_gl660_gl646 (struct usb_device *dev)
 
   /* Check device descriptor */
   if ((dev->descriptor.bDeviceClass != USB_CLASS_VENDOR_SPEC)
-      || (dev->config[0].interface[0].altsetting[0].bInterfaceClass != USB_CLASS_PER_INTERFACE))
+      || (dev->config[0].interface[0].altsetting[0].bInterfaceClass !=
+	  USB_CLASS_PER_INTERFACE))
     {
       if (verbose > 2)
 	printf
@@ -1238,8 +1614,11 @@ check_gl660_gl646 (struct usb_device *dev)
        bEndpointAddress != 0x81)
       || (dev->config[0].interface[0].altsetting[0].endpoint[0].
 	  bmAttributes != 0x02)
-      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
-	  wMaxPacketSize != 0x40)
+      ||
+      ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+	wMaxPacketSize != 0x40)
+       && (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize != 0x200))
       || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
 	  0x0))
     {
@@ -1260,8 +1639,11 @@ check_gl660_gl646 (struct usb_device *dev)
        bEndpointAddress != 0x02)
       || (dev->config[0].interface[0].altsetting[0].endpoint[1].
 	  bmAttributes != 0x02)
-      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
-	  wMaxPacketSize != 0x40)
+      ||
+      ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+	wMaxPacketSize != 0x40)
+       && (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize != 0x200))
       || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
 	  0))
     {
@@ -1282,8 +1664,11 @@ check_gl660_gl646 (struct usb_device *dev)
        bEndpointAddress != 0x83)
       || (dev->config[0].interface[0].altsetting[0].endpoint[2].
 	  bmAttributes != 0x03)
-      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
-	  wMaxPacketSize != 0x1)
+      ||
+      ((dev->config[0].interface[0].altsetting[0].endpoint[2].
+	wMaxPacketSize != 0x1)
+       && (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize != 0x200))
       || (dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval !=
 	  8))
     {
@@ -1334,13 +1719,359 @@ check_gl660_gl646 (struct usb_device *dev)
 }
 
 
+/********** the gl841 section **********/
+
+static char *
+check_gl841 (struct usb_device *dev)
+{
+  unsigned char val;
+  int result;
+  usb_dev_handle *handle;
+
+  if (verbose > 2)
+    printf ("    checking for GL841 ...\n");
+
+  /* Check device descriptor */
+  if ((dev->descriptor.bDeviceClass != USB_CLASS_VENDOR_SPEC)
+      || (dev->config[0].interface[0].altsetting[0].bInterfaceClass !=
+	  USB_CLASS_VENDOR_SPEC))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL841 (bDeviceClass = %d, bInterfaceClass = %d)\n",
+	   dev->descriptor.bDeviceClass,
+	   dev->config[0].interface[0].altsetting[0].bInterfaceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x200)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL841 (bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0xff)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL841 (bDeviceSubClass = 0x%x)\n",
+		dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0xff)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL841 (bDeviceProtocol = 0x%x)\n",
+		dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 3)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL841 (bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+       bEndpointAddress != 0x81)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  bmAttributes != 0x02)
+      || ((dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize != 0x40) &&
+	  (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize != 0x200))
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval !=
+	  0x0))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL841 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+       bEndpointAddress != 0x02)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	  bmAttributes != 0x02)
+      || ((dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize != 0x40) &&
+	  (dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize != 0x200))
+      || (dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval !=
+	  0))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL841 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[1].bInterval);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[0].endpoint[2].
+       bEndpointAddress != 0x83)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  bmAttributes != 0x03)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[2].
+	  wMaxPacketSize != 0x1)
+      ||
+      ((dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval != 8)
+       && (dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval !=
+	   16)))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a GL841 (bEndpointAddress = 0x%x, bmAttributes = 0x%x, "
+	   "wMaxPacketSize = 0x%x, bInterval = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   bEndpointAddress,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bmAttributes,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].
+	   wMaxPacketSize,
+	   dev->config[0].interface[0].altsetting[0].endpoint[2].bInterval);
+      return 0;
+    }
+
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "GL841?";
+
+  result = gl646_write_reg (handle, 0x38, 0x15);
+  if (!result)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL841 (writing register failed)\n");
+      finish_interface (handle);
+      return 0;
+    }
+
+  result = gl646_read_reg (handle, 0x38, &val);
+  if (!result)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL841 (reading register failed)\n");
+      finish_interface (handle);
+      return 0;
+    }
+
+  if (val != 0x15)
+    {
+      if (verbose > 2)
+	printf ("    this is not a GL841 (reg 0x38 != 0x15)\n");
+      finish_interface (handle);
+      return 0;
+    }
+  finish_interface (handle);
+  return "GL841";
+}
+
+
+/********** the icm532b section version 0.2 **********/
+/*          no write and read test registers yet     */
+
+static char *
+check_icm532b (struct usb_device *dev)
+{
+  int result;
+  usb_dev_handle *handle;
+
+  if (verbose > 2)
+    printf ("    checking for ICM532B ...\n");
+
+  /* Check device descriptor */
+  if ((dev->descriptor.bDeviceClass != USB_CLASS_VENDOR_SPEC)
+      || (dev->config[0].interface[0].altsetting[0].bInterfaceClass != 0xff))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 1, bDeviceClass = %d, bInterfaceClass = %d)\n",
+	   dev->descriptor.bDeviceClass,
+	   dev->config[0].interface[0].altsetting[0].bInterfaceClass);
+      return 0;
+    }
+  if (dev->descriptor.bcdUSB != 0x110)
+    {
+      if (verbose > 2)
+	printf ("    this is not a ICM532B (check 2, bcdUSB = 0x%x)\n",
+		dev->descriptor.bcdUSB);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceSubClass != 0xff)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 3, bDeviceSubClass = 0x%x)\n",
+	   dev->descriptor.bDeviceSubClass);
+      return 0;
+    }
+  if (dev->descriptor.bDeviceProtocol != 0xff)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 4, bDeviceProtocol = 0x%x)\n",
+	   dev->descriptor.bDeviceProtocol);
+      return 0;
+    }
+
+  /* Check endpoints */
+  if (dev->config[0].interface[0].altsetting[0].bNumEndpoints != 0x01)
+    {
+      if (verbose > 2)
+	printf ("    this is not a ICM532B (check 5, bNumEndpoints = %d)\n",
+		dev->config[0].interface[0].altsetting[0].bNumEndpoints);
+      return 0;
+    }
+  /* Check bEndpointAddress */
+  if (dev->config[0].interface[0].altsetting[0].endpoint[0].
+      bEndpointAddress != 0x81)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 6, bEndpointAddress = %d)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bEndpointAddress);
+      return 0;
+    }
+  /* Check bmAttributes */
+  if (dev->config[0].interface[0].altsetting[0].endpoint[0].
+      bmAttributes != 0x01)
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 7, bEndpointAddress = %d)\n",
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   bmAttributes);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[0].bAlternateSetting != 0x00)
+      || (dev->config[0].interface[0].altsetting[0].endpoint[0].
+	  wMaxPacketSize != 0x00))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 8, bAlternateSetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[0].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[0].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+
+  if ((dev->config[0].interface[0].altsetting[1].bAlternateSetting != 0x01)
+      || (dev->config[0].interface[0].altsetting[1].endpoint[0].
+	  wMaxPacketSize != 0x100))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 9, bAlternatesetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[1].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[1].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[2].bAlternateSetting != 0x02)
+      || (dev->config[0].interface[0].altsetting[2].endpoint[0].
+	  wMaxPacketSize != 0x180))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 10, bAlternatesetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[2].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[2].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[3].bAlternateSetting != 0x03)
+      || (dev->config[0].interface[0].altsetting[3].endpoint[0].
+	  wMaxPacketSize != 0x200))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 11, bAlternatesetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[3].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[3].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[4].bAlternateSetting != 0x04)
+      || (dev->config[0].interface[0].altsetting[4].endpoint[0].
+	  wMaxPacketSize != 0x280))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 12, bAlternatesetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[4].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[4].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[5].bAlternateSetting != 0x05)
+      || (dev->config[0].interface[0].altsetting[5].endpoint[0].
+	  wMaxPacketSize != 0x300))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 13, bAlternatesetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[5].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[5].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[6].bAlternateSetting != 0x06)
+      || (dev->config[0].interface[0].altsetting[6].endpoint[0].
+	  wMaxPacketSize != 0x380))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 14, bAlternatesetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[6].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[6].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+  if ((dev->config[0].interface[0].altsetting[7].bAlternateSetting != 0x07)
+      || (dev->config[0].interface[0].altsetting[7].endpoint[0].
+	  wMaxPacketSize != 0x3ff))
+    {
+      if (verbose > 2)
+	printf
+	  ("    this is not a ICM532B (check 15, bAlternatesetting = 0x%x, wMaxPacketSize = 0x%x)\n",
+	   dev->config[0].interface[0].altsetting[7].bAlternateSetting,
+	   dev->config[0].interface[0].altsetting[7].endpoint[0].
+	   wMaxPacketSize);
+      return 0;
+    }
+
+  result = prepare_interface (dev, &handle);
+  if (!result)
+    return "ICM532B?";
+
+  finish_interface (handle);
+  return "ICM532B";
+}
+
+/* ====================================== end of icm532b ==================*/
 
 char *
-check_usb_chip (struct usb_device *dev, int verbosity)
+check_usb_chip (struct usb_device *dev, int verbosity, SANE_Bool from_file)
 {
   char *chip_name = 0;
 
   verbose = verbosity;
+  no_chipset_access = from_file;
 
   if (verbose > 2)
     printf ("\n<trying to find out which USB chip is used>\n");
@@ -1350,6 +2081,9 @@ check_usb_chip (struct usb_device *dev, int verbosity)
   if (!chip_name)
     chip_name = check_gt6816 (dev);
 
+  if (!chip_name)
+    chip_name = check_gt8911 (dev);
+
   if (!chip_name)
     chip_name = check_ma1017 (dev);
 
@@ -1365,9 +2099,18 @@ check_usb_chip (struct usb_device *dev, int verbosity)
   if (!chip_name)
     chip_name = check_gl646 (dev);
 
+  if (!chip_name)
+    chip_name = check_gl646_hp (dev);
+
   if (!chip_name)
     chip_name = check_gl660_gl646 (dev);
 
+  if (!chip_name)
+    chip_name = check_gl841 (dev);
+
+  if (!chip_name)
+    chip_name = check_icm532b (dev);
+
   if (verbose > 2)
     {
       if (chip_name)
